﻿using System.Security;
using System.Text.RegularExpressions;

namespace TestSessionLib
{
    public static class REG_MARK_LIB
    {
        /// <summary>
        /// Список символов, которые можно использовать в серии номера.
        /// </summary>
        static List<char> chars = new() { 'a', 'b', 'e' , 'k', 'm', 'n',
            'o', 'p', 'c', 't', 'y', 'x'};

        /// <summary>
        /// Проверяет переданный номерной знак в формате a999aa999 (латинскими буквами)
        /// </summary>
        /// <param name="mark">Регистрационный номер.</param>
        /// <returns>true или false в зависимости от правильности номерного знака</returns>
        public static Boolean CheckMark(String mark)
        {
            bool result = true;
            //проверка строки через регулярное выражение
            Regex regex = new Regex(@"^[a,b,e,k,m,n,o,p,c,t,y,x]\d\d\d[a,b,e,k,m,n,o,p,c,t,y,x][a,b,e,k,m,n,o,p,c,t,y,x]\d\d\d$");
            result = regex.Match(mark).Success;
            if (result == false)
                return result;
            //проверка региона после общей проверки 
            // в файле "code_region.txt" хранится список регионов
            int[] codeRegion = Array.ConvertAll(File.ReadAllLines("code_region.txt"), s => int.Parse(s));
            int region = int.Parse($"{mark[6]}{mark[7]}{mark[8]}");
            result = codeRegion.Contains(region);
            return result;
        }

        /// <summary>
        /// Выдает следующий номер, после требуеомого, в данной серии или создает следующую серию.
        /// </summary>
        /// <param name="mark">Регистрационный знак.</param>
        /// <returns>Следующий номер после требуемого.</returns>
        /// <exception cref="ArgumentException">Ошибка входных данных.</exception>
        public static String GetNextMarkAfter(String mark)
        {
            if(!CheckMark(mark))
            {
                throw new ArgumentException("Wrong license plate");
            }
            string result = null;
            //получение числа 
            int number = int.Parse($"{mark[1]}{mark[2]}{mark[3]}");
            if (number != 999)
            {
                number++;
                result = mark[0] + string.Format("{0:d3}", number) + mark.Substring(4);
            }
            else
            {
                char[] charResult = mark.Clone().ToString().ToCharArray();
                charResult[1] = '0';
                charResult[2] = '0';
                charResult[3] = '0';
                if (mark[5] != chars.Last())
                {
                    charResult[5] = chars[chars.IndexOf(mark[5]) + 1];
                }
                else
                {
                    if (mark[4] != chars.Last())
                    {
                        charResult[4] = chars[chars.IndexOf(mark[4]) + 1];
                        charResult[5] = chars.First();

                    }
                    else
                    {
                        charResult[0] = chars[chars.IndexOf(mark[0]) + 1];
                        charResult[4] = chars.First();
                        charResult[5] = chars.First();
                    }
                }
                result = new string(charResult);
            }
            return result;
        }

        /// <summary>
        /// Выдает следующий номер в данном промежутке номеров rangeStart до rangeEnd (включая обе границы).
        /// </summary>
        /// <param name="prevMark">Текущий регситрационный номер.</param>
        /// <param name="rangeStart">Начало диапозона номеров.</param>
        /// <param name="rangeEnd">Конец диапозона номеров.</param>
        /// <returns>Следующий номер в данной данном промежутке номеров.</returns>
        /// <exception cref="ArgumentException">Ошибка входных данных.</exception>
        public static String GetNextMarkAfterInRange(String prevMark, String rangeStart, String rangeEnd)
        {
            if (!CheckMark(prevMark) )
            {
                throw new ArgumentException("Wrong license plate previous Mark");
            }
            if (!CheckMark(rangeStart) )
            {
                throw new ArgumentException("Wrong license plate range start Mark");
            }
            if (!CheckMark(rangeEnd) )
            {
                throw new ArgumentException("Wrong license plate range end Mark");
            }
            if (string.Compare(rangeStart, rangeEnd) > 0)
                throw new ArgumentOutOfRangeException("The range is not order by alphabet.");

            string result = "out of stock";

            if (string.Compare(rangeStart, prevMark) > 0 || string.Compare(rangeEnd, prevMark) < 0)
                return result;
            if (rangeStart.Contains(rangeEnd) || rangeStart.Remove(0, 6) != rangeEnd.Remove(0, 6))
                return result;
            result = GetNextMarkAfter(prevMark);

            return result;
        }

        /// <summary>
        /// Количество возможных номеров в диапозоне.
        /// </summary>
        /// <param name="mark1">Начало диапозона.</param>
        /// <param name="mark2">Конец диапозона.</param>
        /// <returns>Число возможных номеров.</returns>
        /// <exception cref="ArgumentException">Ошибка входных данных.</exception>
        public static int GetCombinationsCountInRange(String mark1, String mark2)
        {
            if (!CheckMark(mark1))
            {
                throw new ArgumentException("Wrong license plate range start Mark");
            }
            if (!CheckMark(mark2))
            {
                throw new ArgumentException("Wrong license plate range end Mark");
            }
            if (string.Compare(mark1, mark2) > 0)
                throw new ArgumentOutOfRangeException("The range is not order by alphabet.");

            string nextMark = mark1;
            //counter = 1, т.к. границы также входят в счет
            int counter = 1;
            while (!mark2.Contains(nextMark))
            {
                counter++;
                nextMark = GetNextMarkAfter(nextMark);
            }
            return counter;
        }
    }
}